//+------------------------------------------------------------------+
//|                                                     C_Router.mqh |
//|                                                      Daniel Jose |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Daniel Jose"
//+------------------------------------------------------------------+
#include <Auxiliar\C_Terminal.mqh>
//+------------------------------------------------------------------+
class C_Router
{
	protected:
		double				m_VolMinimal;
		MqlTradeRequest	TradeRequest;
		MqlTradeResult		TradeResult;
	private  :
		ulong		m_Ticket;
		bool		m_IsBuy;
		double	m_Take, m_Stop, m_Volume;
//+------------------------------------------------------------------+
		bool Send(void)
			{
				if (!OrderSend(TradeRequest, TradeResult))
				{
					MessageBox(StringFormat("Error Number: %d", TradeResult.retcode), "Router");
					return false;
				}
				return true;
			}
//+------------------------------------------------------------------+
		bool ClosePosition(const int arg = 0)
			{
				double v1 = arg * m_VolMinimal;
				
				if(!PositionSelectByTicket(m_Ticket)) return false;
				ZeroMemory(TradeRequest);
				ZeroMemory(TradeResult);
				TradeRequest.action		= TRADE_ACTION_DEAL;
				TradeRequest.type			= (m_IsBuy ? ORDER_TYPE_SELL : ORDER_TYPE_BUY);
				TradeRequest.price		= SymbolInfoDouble(Terminal.GetSymbol(), (m_IsBuy ? SYMBOL_BID : SYMBOL_ASK));
				TradeRequest.position	= m_Ticket;
				TradeRequest.symbol		= Terminal.GetSymbol();
				TradeRequest.volume		= ((v1 == 0) || (v1 > m_Volume) ? m_Volume : v1);
				TradeRequest.deviation	= 1000;
				if (!Send()) return false;
				m_Ticket = 0;
				
				return true;
			};
//+------------------------------------------------------------------+
	public   :
		C_Router() : m_Ticket(0), m_IsBuy(true), m_Take(0.0), m_Stop(0.0), m_Volume(0.0) {};
//+------------------------------------------------------------------+
		void UpdatePosition(void)
			{
				for(int i0 = PositionsTotal() - 1; i0 >= 0; i0--) if(PositionGetSymbol(i0) == Terminal.GetSymbol())
				{
					m_Take		= PositionGetDouble(POSITION_TP);
					m_Stop		= PositionGetDouble(POSITION_SL);
					m_IsBuy		= PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY;
					m_Volume		= PositionGetDouble(POSITION_VOLUME);
					m_Ticket		= PositionGetInteger(POSITION_TICKET);
				}
			};
//+------------------------------------------------------------------+
		inline bool CheckPosition(const double price = 0, const int factor = 0)
			{
				double last;

				if(m_Ticket == 0) return false;
				last = SymbolInfoDouble(Terminal.GetSymbol(), SYMBOL_LAST);
				if(m_IsBuy)
				{
					if((last > m_Take) || (last < m_Stop)) return ClosePosition();
					if((price > 0) && (price >= last)) return ClosePosition(factor);
				} else
				{
					if((last < m_Take) || (last > m_Stop)) return ClosePosition();
					if((price > 0) && (price <= last)) return ClosePosition(factor);
				}

				return false;
			};
//+------------------------------------------------------------------+
		ulong CreateOrderPendent(const bool IsBuy, const double Volume, const double Price, const double Take, const double Stop, const bool DayTrade = true)
			{
				double last = SymbolInfoDouble(Terminal.GetSymbol(), SYMBOL_LAST);
				
				if (OrdersTotal() > 0) return 0;
				ZeroMemory(TradeRequest);
				ZeroMemory(TradeResult);
				TradeRequest.action			= TRADE_ACTION_PENDING;
				TradeRequest.symbol			= Terminal.GetSymbol();
				TradeRequest.volume			= Volume;
				TradeRequest.type				= (IsBuy ? (last >= Price ? ORDER_TYPE_BUY_LIMIT : ORDER_TYPE_BUY_STOP) : (last < Price ? ORDER_TYPE_SELL_LIMIT : ORDER_TYPE_SELL_STOP));
				TradeRequest.price			= NormalizeDouble(Price, (int) SymbolInfoInteger(Terminal.GetSymbol(), SYMBOL_DIGITS));
				TradeRequest.sl				= NormalizeDouble(Stop, (int) SymbolInfoInteger(Terminal.GetSymbol(), SYMBOL_DIGITS));
				TradeRequest.tp				= NormalizeDouble(Take, (int) SymbolInfoInteger(Terminal.GetSymbol(), SYMBOL_DIGITS));
				TradeRequest.type_time		= (DayTrade ? ORDER_TIME_DAY : ORDER_TIME_GTC);
				TradeRequest.stoplimit		= 0;
				TradeRequest.expiration		= 0;
				TradeRequest.type_filling	= ORDER_FILLING_RETURN;
				TradeRequest.deviation		= 1000;
				TradeRequest.comment			= "Order Generated by Experts Advisor.";
				if (!Send()) return 0;
								
				return TradeResult.order;
			};
//+------------------------------------------------------------------+
};
//+------------------------------------------------------------------+
